      program example

      implicit none

      include 'HB.h'

c     %------------%
c     | Parameters |
c     %------------%
      integer lwrk,liwrk
      parameter(lwrk=mmax+nmax+13*kmax+8*kmax**2+32*mmax+8)
      parameter(liwrk = 8*kmax)
      double precision dzero, done
      parameter(dzero = 0.0, done = 1.0)

      character*256 filename
      integer i,m,n,neig,ioption(10),iwork(liwrk)
      integer iparm(1),info
      double precision U(mmax,kmax+1),V(nmax,kmax),work(lwrk)
      double precision doption(10),Sigma(kmax),dparm(1)
      double precision tol,bnd(kmax)
c     Save large arrays in a common block to avoid stack overflow on
c     certain machines.
      common/cexample/U,V,Sigma,work,iwork

c     %--------------------%
c     | External Functions |
c     %--------------------%
      external hbmv

c-------------------- Here begins executable code ---------------------
      
c
c     Get filename and number of singular values from stdin.
c
      write (*,*) 'Name of file with matrix in Harwell-Boeing',
     c     ' format?'
      read (5,'(a)') filename
      write(*,*) filename

      write (*,*) 'Desired number of singular values?'
      read (5,*) neig
      write (*,*) neig

c
c     Read matrix in Harwell-Boeing format.
c
      write (*,*) 'Reading matrix from file ',filename
      open(10,file=filename,status='old')
      call readHB(10,m,n)
      close(10)
      write (*,*) '...done'
      write (*,*) 'Dimensions of matrix: m,n = ',m,n
      if (m.gt.mmax .or. n.gt.nmax) then
         write (*,'(a, I6, a, i6)') 'Matrix can be at most',mmax,' by ',
     c        nmax
         stop 'Matrix dimensions exceed allocated size'
      endif
         
c
c     Set the first column of U to zero. This will make dlansvd_irl
c     generate a random starting vector.      
c
      do i=1,m
         U(i,1) = 1d0
      enddo

c
c     Clear statistics counters and timers
c
      call clearstat

c
c     Set options for DLANSVD
c
      ioption(1) = 0
      ioption(2) = 1
      doption(1) = 1d-8
      doption(2) = 1e-12
      doption(3) = dzero
      tol = 1d-12
c
c     Compute the neig largest singular values
c
      write (*,*) 'Computing the neig=',neig,' largest singular ' //
     c     'values and corresponding singular vectors'
      call dlansvd('y','y',m,n,neig,kmax,hbmv,U,mmax,Sigma,bnd,V,nmax,
     c     tol,work,lwrk,iwork,liwrk,doption,ioption,info,dparm,iparm)

c
c     Print singular values and computed error bounds.
c
      write(*,*)
      write(*,*) '  i          sigma(i)              bnd(i)'
      do i=1,neig
         write(*,'(1I5,1p,1e25.15,1p,1e12.3)') i,Sigma(i),bnd(i)
      enddo 
c
c     Print a few entries from the first and last, left and right
c     singular vectors.
c
      write(*,*)
      write(*,*) 'A few entries from selected singular vectors:'
      write(*,'(a11,1p,1e25.15)') 'U(1,1)   : ', U(1,1)
      write(*,'(a11,1p,1e25.15)') 'U(m,1)   : ', U(m,1)
      write(*,'(a11,1p,1e25.15)') 'U(1,neig): ', U(1,neig)
      write(*,'(a11,1p,1e25.15)') 'U(m,neig): ', U(n,neig)
      write(*,'(a11,1p,1e25.15)') 'V(1,1)   : ', V(1,1)
      write(*,'(a11,1p,1e25.15)') 'V(n,1)   : ', V(n,1)
      write(*,'(a11,1p,1e25.15)') 'V(1,neig): ', V(1,neig)
      write(*,'(a11,1p,1e25.15)') 'V(n,neig): ', V(n,neig)
      write(*,*)


c
c     Print runtime statistics 
c
      call printstat
      end

